Imports Microsoft.VisualBasic
Imports System
Imports System.Text
Imports System.Collections
Imports System.Text.RegularExpressions
Imports System.Xml

Namespace System.IO

   Public Class PathEx
      'Objekt zum Manipulieren von Dateinamen - bld, da man die Path-Klasse nicht beerben kann
      Private _Fullname As String

      Public Sub New(ByVal FullnameArg As String)
         _Fullname = FullnameArg
      End Sub

      Public Property DirectoryName() As String
         Get
            Return Path.GetDirectoryName(_Fullname)
         End Get
         Set(ByVal Value As String)
            _Fullname = Value.TrimEnd("\"c) & "\" & Path.GetFileName(_Fullname)
         End Set
      End Property

      Public Property FileName() As String
         Get
            Return Path.GetFileName(_Fullname)
         End Get
         Set(ByVal Value As String)
            _Fullname = DirectoryName & "\" & Value
         End Set
      End Property

      Public Property Extension() As String
         Get
            Return Path.GetExtension(_Fullname)
         End Get
         Set(ByVal Value As String)
            _Fullname = String.Concat( _
                StringEx.LastLeftCut(_Fullname, ".", FailReturn:=_Fullname), _
                ".", _
                Value.TrimStart("."c))
         End Set
      End Property

#Region "trivial not shared stuff"

      Public Property Fullname() As String
         Get
            Return _Fullname
         End Get
         Set(ByVal Value As String)
            _Fullname = Value
         End Set
      End Property

      Public Function LeftString(ByVal Length As Integer) As String
         Return Fullname.Substring(0, Length)
      End Function

      Public ReadOnly Property FilenameWithoutExtension() As String
         Get
            Return Path.GetFileNameWithoutExtension(_Fullname)
         End Get
      End Property

      Public ReadOnly Property FullnameWithoutExtension() As String
         Get
            Return FullnameWithoutExtension(_Fullname)
         End Get
      End Property

      Public Function RelativePath(ByVal sAbsolutePath As String) As String
         Return RelativePath(sAbsolutePath, DirectoryName)
      End Function

      Public Function ShortenFullname(ByVal VisibleLevels As Integer) As String
         Return ShortenFullname(_Fullname, VisibleLevels)
      End Function

      Public Function AbsolutePath(ByVal RelativePath$) As String
         Return AbsolutePath(RelativePath, DirectoryName)
      End Function

      Public Function AppendNumber( _
                  Optional ByVal nDigits As Integer = 2, _
                  Optional ByVal sAppend As String = "") As String
         Return AppendNumber(_Fullname, nDigits, sAppend)
      End Function

      Public Function FillGapsInNumberedPathes( _
                  Optional ByVal nDigits As Integer = 2, _
                  Optional ByVal Prompt As Boolean = True, _
                  Optional ByVal sAppend As String = "") As Boolean
         Return FillGapsInNumberedPathes(_Fullname, nDigits, Prompt, sAppend)
      End Function
#End Region 'trivial not shared stuff

#Region "shared stuff"

      Public Shared Function FillGapsInNumberedPathes( _
                  ByVal FullName As String, _
                  Optional ByVal nDigits As Integer = 2, _
                  Optional ByVal Prompt As Boolean = True, _
                  Optional ByVal sAppend As String = "") As Boolean
         Return EnumerateNumberedPathes(FullName, True, nDigits, Prompt, sAppend) = ""
      End Function

      Public Shared Function EnumerateNumberedPathes( _
                  ByVal FullName As String, _
                  ByVal bFillGapsMode As Boolean, _
                  Optional ByVal nDigits As Integer = 2, _
                  Optional ByVal bMustConfirmFillGaps As Boolean = True, _
                  Optional ByVal sAppend As String = "") As String
         'gibt einen mit Orndungsnummer verzierten Pfad zurck
         'falls mit bFillGapsMode=True aufgerufen, fhrt er ggfs. Umnumerierungen der Files durch
         Dim NewFullName As String = FullnameWithoutExtension(FullName) & sAppend
         Dim Ext As String = Path.GetExtension(FullName)
         Dim MaxNum As Integer = 0
         Dim Pathes() As String = Directory.GetFileSystemEntries( _
             Path.GetDirectoryName(FullName), _
             Path.GetFileName(NewFullName) & Strings.StrDup(nDigits, "?"c) & Ext)
         Array.Sort(Pathes)
         For Each sName As String In Pathes
            If NewFullName.Length + nDigits > sName.Length Then Continue For
            Dim sNumb As String = sName.Substring(NewFullName.Length, nDigits)
            If Not IsNumeric(sNumb) Then Continue For
            Dim FileNumb As Integer = Integer.Parse(sNumb)
            Select Case FileNumb
               Case MaxNum
                  MaxNum += 1
               Case Is > MaxNum                    'Lcke in Numerierung der durchnumerierten Pfade
                  If bFillGapsMode Then   'diesen und die folgenden Pfade ggfs. umbenennen, um die
                     '!                                                        Lcke zu schlieen
                     If bMustConfirmFillGaps Then
                        If ConfirmFillGaps(sName) Then
                           bMustConfirmFillGaps = False  'Besttigung der Umbenennerei gilt fr
                           '!                                 alle folgenden Numerierung-Lcken
                        Else
                           Return ""
                        End If
                     End If
                     File.Move( _
                         sName, _
                         NewFullName & MaxNum.ToString("0#") & Ext)
                     FileNumb = MaxNum
                  End If
                  MaxNum = FileNumb + 1
            End Select
         Next
         Return NewFullName & MaxNum.ToString("0#") & Ext
      End Function

      Private Shared Function ConfirmFillGaps(ByVal sName As String) As Boolean
         With New Text.StringBuilder
            .Append("Gap detected in numbered Pathes before:").Append(vbNewLine)
            .Append(sName).Append(vbNewLine)
            .Append("Shall I fill them by renaming the ""wrong"" named files?")
            Return MsgBox(.ToString, MsgBoxStyle.YesNoCancel) = MsgBoxResult.Yes
         End With
      End Function

      Public Shared Sub PreparePath( _
              ByRef sPath As String, Optional ByVal AddSeparator As Boolean = False)
         sPath = PreparedPath(sPath, AddSeparator)
      End Sub

      Public Shared Function PreparedPath( _
              ByVal sPath As String, Optional ByVal AddSeparator As Boolean = False) As String
         If AddSeparator Then
            Return sPath.TrimEnd("\"c, " "c) & "\"c
         Else
            Return sPath.TrimEnd("\"c, " "c)
         End If
      End Function


      Public Enum GetCommonPathResult As Integer
         NoPathesGiven
         NoneFound
         GuessFileOrFolder
         Success
      End Enum

      Public Shared Function GetCommonPathLength( _
              ByVal FilesystemItems() As Object, _
              ByRef CommonPathLength As Integer) As GetCommonPathResult
         'finde die Lnge des Namens des Ordners, der alle FilesystemItems enthlt
         Select Case FilesystemItems.Length
            Case 0
               Return GetCommonPathResult.NoPathesGiven
            Case 1  'Ist genau 1 Element in FilesystemItems enthalten, so bildet dieses komplett den
               '!                           CommonPath, und man wei nicht: isses File oder Folder?
               CommonPathLength = CStr(FilesystemItems(0)).Length
               Return GetCommonPathResult.GuessFileOrFolder
            Case Else
               Dim sCompare As String = CStr(FilesystemItems(0))
               CommonPathLength = Integer.MaxValue
               For Each FilesystemItem As String In FilesystemItems
                  CommonPathLength = Math.Min(CommonPathLength, FilesystemItem.Length)
                  Do Until String.Compare( _
                          sCompare.Substring(0, CommonPathLength), _
                          FilesystemItem.Substring(0, _
                          CommonPathLength), ignoreCase:=True) = 0
                     CommonPathLength = _
                         sCompare.LastIndexOf("\"c, CommonPathLength - 1, CommonPathLength)
                     If CommonPathLength < 0 Then Return GetCommonPathResult.NoneFound
                  Loop
               Next
               Return GetCommonPathResult.Success
         End Select
      End Function

      Public Shared Function GetCommonPath( _
              ByVal FilesystemItems() As Object, _
              ByRef CommonPath As String) As GetCommonPathResult
         Dim CommonPathLength As Integer
         GetCommonPath = GetCommonPathLength(FilesystemItems, CommonPathLength)
         If GetCommonPath > GetCommonPathResult.NoneFound Then
            CommonPath = CStr(FilesystemItems(0)).Substring(0, CommonPathLength)
         End If
      End Function

      Public Shared ReadOnly Property FullnameWithoutExtension(ByVal _Fullname As String) As String
         Get
            Return String.Concat( _
                Path.GetDirectoryName(_Fullname), _
                "\", _
                Path.GetFileNameWithoutExtension(_Fullname))
         End Get
      End Property


      Public Shared Function AbsolutePath(ByVal sRelativePath As String, ByVal RelativeTo As String) As String
         Dim Start As Integer
         While sRelativePath.Substring(Start).StartsWith("..\")
            RelativeTo = Path.GetDirectoryName(RelativeTo)
            Start += 3
         End While
         Return RelativeTo & "\" & sRelativePath.Substring(Start)
      End Function


      Public Shared Function RelativePath(ByVal sAbsolutePath$, ByVal sRoot$) As String
         Dim CommonPathLen, I As Integer
         PreparePath(sRoot)
         PreparePath(sAbsolutePath)
         If GetCommonPathLength( _
                 New String() {sRoot, sAbsolutePath}, CommonPathLen) <> GetCommonPathResult.Success Then
            Dim Msg As String = String.Format( _
                "kann keinen gemeinsamen Pfad finden, fr {0}'{1}'{2}und{3}'{4}'", _
                vbNewLine, sRoot, vbNewLine, vbNewLine, sAbsolutePath)
            Throw New Exception(Msg)
         End If
         RelativePath = sAbsolutePath.Substring(CommonPathLen + 1)  'gemeinsamen fhrenden Teilstring
         '!                                                                                 entfernen
         'Falls sAbsolutePath gar kein Subfolder von sRoot ist, die entsprechenden "..\" davorsetzen
         For I = 1 To StringEx.CountChars(sRoot.Substring(CommonPathLen), "\"c)
            RelativePath = "..\" & RelativePath
         Next
      End Function

      Public Shared Function ShortenFullname( _
              ByVal Fullname As String, ByVal VisibleLevels As Integer) As String
         'haut die ersten Segmente eines Pfades weg, lt die letzten VisibleLevels Segmente stehen.
         Dim Splitted() As String = Fullname.Split("\"c)
         Dim FirstVisible As Integer = Splitted.Length - VisibleLevels
         Return "...\" & String.Join("\", Splitted, FirstVisible, VisibleLevels)
      End Function


      Public Shared Function AppendNumber( _
              ByVal Fullname As String, _
              Optional ByVal nDigits As Integer = 2, _
              Optional ByVal sAppend As String = "") As String
         Return EnumerateNumberedPathes(Fullname, False, nDigits, False, sAppend)
      End Function

      Public Shared Sub CopyForced(ByVal Src As String, ByVal Dst As String)
         'kopieren ohne Gnade, Folder oder Files; bestehendes wird berschrieben, nichtexistente
         '! Folder creiert, Folder rekursiv.
         If File.Exists(Src) Then
            Dim sDest As String = Path.GetDirectoryName(Dst)
            If Not Directory.Exists(sDest) Then Directory.CreateDirectory(sDest)
            File.Copy(Src, Dst, overwrite:=True)
         ElseIf Directory.Exists(Src) Then
            If File.Exists(Dst) Then Throw New Exception( _
                    Dst & vbNewLine & "kann nicht erstellt werden, da ein gleichnamiges File existiert")
            CopyFolder(Src, Dst)
         Else
            Throw New Exception("Pfad nicht gefunden:" & vbNewLine & Src)
         End If
      End Sub

      Public Shared Sub CopyFolder(ByVal sSrc As String, ByVal sDest As String)
         'rekursiv alle Files eines Folders herumkopieren
         For Each FullName As String In Directory.GetDirectories(sSrc)
            CopyFolder(FullName, sDest & "\" & Path.GetFileName(FullName))
         Next
         If Not Directory.Exists(sDest) Then Directory.CreateDirectory(sDest)
         For Each FullName As String In Directory.GetFiles(sSrc)
            File.Copy(FullName, sDest & "\" & Path.GetFileName(FullName), overwrite:=True)
         Next
      End Sub

      Public Shared Function UpperDirectory(ByVal Path As String, ByVal Count As Integer) As String
         If Count < 0 Then
            Throw New ArgumentOutOfRangeException("Count", Count, "darf nicht kleiner 0 werden")
         End If
         For i As Integer = 1 To Count
            Path = System.IO.Path.GetDirectoryName(Path)
         Next
         Return Path
      End Function

#End Region 'shared stuff
   End Class 'PathEx
End Namespace
