Imports Microsoft.VisualBasic
Imports System
Imports System.io


Namespace System.Windows.Forms

   Public Class clsTreeNodeEx
      'ntzliche Erweiterungen von TreeNode
      Inherits TreeNode

      'Public Event Grounded(ByVal sender As Object, ByVal e As EventArgs)

#Region "Vars"
      Protected WithEvents _TV As TreeView
#End Region 'Vars

#Region "Initiation"
      Public Sub New(ByVal Text As String)
         MyBase.New(Text)
      End Sub

      Public Sub New()
         MyBase.New()
      End Sub
#End Region 'Initiation

#Region "public Members"
      Public ReadOnly Property Instance() As clsTreeNodeEx
         Get
            Return Me
         End Get
      End Property

      Public Function AddTo( _
          ByVal ParentNode As TreeNode) As clsTreeNodeEx
         Return AddTo(ParentNode.Nodes)
      End Function

      Public Overridable Function AddTo( _
          ByVal ParentNodes As TreeNodeCollection) As clsTreeNodeEx
         ParentNodes.Add(Me)
         If Me.TreeView IsNot Nothing Then
            OnTreeviewChanged()
         End If
         Return Me
      End Function

      Public ReadOnly Property IsTopLevel() As Boolean
         Get
            Return Me.Parent Is Nothing
         End Get
      End Property

      Public Shadows ReadOnly Property Parent() As clsTreeNodeEx
         Get
            Return DirectCast(MyBase.Parent, clsTreeNodeEx)
         End Get
      End Property

      Public Function GetNodeUnderMouse() As TreeNode
         Return MyBase.TreeView.GetNodeAt(MyBase.TreeView.PointToClient(Control.MousePosition))
      End Function

      Public Function GetNodeAtScreenPos( _
              ByVal X As Integer, ByVal Y As Integer) As TreeNode
         Return GetNodeAtScreenPos(MyBase.TreeView, X, Y)
      End Function

      Public Function GetNodeByText( _
              ByVal sText As String, _
              Optional ByVal ThrowError As Boolean = True) As clsTreeNodeEx
         Return DirectCast(GetByObject(MyBase.Nodes, sText, "Text", ThrowError), clsTreeNodeEx)
      End Function

      Public Overridable Sub Delete(Optional ByVal Recursive As Boolean = False)
         If Not Recursive Then
            'nur diese Instanz ausschneiden, Children an meinen Parent hngen
            Dim ParentNodes As TreeNodeCollection
            If Me.Parent Is Nothing Then
               ParentNodes = Me.TreeView.Nodes
            Else
               ParentNodes = Me.Parent.Nodes
            End If
            While Me.GetNodeCount(False) > 0
               With DirectCast(Me.Nodes(0), clsTreeNodeEx)
                  .Remove()
                  .AddTo(ParentNodes)
               End With
            End While
         End If
         Me.Remove()
         If _TV IsNot Nothing Then OnTreeviewChanged()
      End Sub

      Public Overloads Sub EnsureVisible( _
              ByVal Align As System.Drawing.StringAlignment, Optional ByVal Forced As Boolean = False)
         If MyBase.IsVisible AndAlso Not Forced Then Return
         'Zeigt den sichtbar zu machenden Node Oben / Unten / Mitten im Treeview an.
         Select Case Align
            Case StringAlignment.Near
               If MyBase.Bounds.Top > 0 Then
                  'letzten Node des Treeview aufsuchen und sichtbar machen
                  Dim Nodes As TreeNodeCollection = MyBase.TreeView.Nodes
                  Dim ndLast As TreeNode = Nodes(Nodes.Count - 1)
                  Do While ndLast.IsExpanded
                     Nodes = ndLast.Nodes
                     If Nodes.Count > 0 Then
                        ndLast = ndLast.Nodes(ndLast.Nodes.Count - 1)
                     Else
                        Exit Do
                     End If
                  Loop
                  ndLast.EnsureVisible()
               End If

            Case StringAlignment.Far
               MyBase.TreeView.Nodes(0).EnsureVisible()

            Case StringAlignment.Center
               Dim Distance As Integer = (MyBase.TreeView.VisibleCount * MyBase.Bounds.Height) \ 2
               Dim TopNodeTop As Integer = MyBase.Bounds.Top - Distance

               Dim AllNodes As New clsTreeEnumerable(MyBase.TreeView.Nodes, "Nodes")
               For Each ndTop As clsTreeNodeEx In AllNodes
                  If ndTop.Bounds.Top >= TopNodeTop Then
                     ndTop.EnsureVisible(StringAlignment.Near, True)
                     Return
                  End If
                  If Not ndTop.IsExpanded Then
                     AllNodes.Skip = True
                  End If
               Next

         End Select
         MyBase.EnsureVisible()

      End Sub

#End Region 'public Members

#Region "non public Members"

      Protected Overridable Sub OnTreeviewChanged()
         _TV = Me.TreeView
         For Each Nd As clsTreeNodeEx In Me.Nodes
            Nd.OnTreeviewChanged()
         Next
         'RaiseEvent Grounded(Me, New EventArgs)
      End Sub

#End Region 'non public Members

#Region "Public Shared Stuff"
      Public Shared Function Create(ByVal Text As String) As clsTreeNodeEx
         Return New clsTreeNodeEx(Text)
      End Function

      Public Shared Function GetNodeAtScreenPos( _
              ByVal TV As TreeView, ByVal X As Integer, ByVal Y As Integer) As TreeNode
         Return TV.GetNodeAt(TV.PointToClient(New Point(X, Y)))
      End Function

      Public Shared Function GetNodeAtScreenPos( _
              ByVal TV As TreeView, ByVal Pt As Point) As TreeNode
         Return TV.GetNodeAt(TV.PointToClient(Pt))
      End Function

      Public Shared Function GetNodeUnderMouse(ByVal TV As TreeView) As TreeNode
         Return TV.GetNodeAt(TV.PointToClient(Control.MousePosition))
      End Function

      Public Shared Function GetNodeExByText( _
              ByVal Nodes As TreeNodeCollection, ByVal sText As String, _
              Optional ByVal ThrowError As Boolean = True) As clsTreeNodeEx
         Return DirectCast(GetByObject(Nodes, sText, "Text", ThrowError), clsTreeNodeEx)
      End Function

      Public Shared Function GetNodeExByPath( _
              ByVal Nodes As TreeNodeCollection, ByVal sPath As String, _
              Optional ByVal ThrowError As Boolean = True) As clsTreeNodeEx
         Return DirectCast(GetNodeByPath(Nodes, sPath, ThrowError), clsTreeNodeEx)
      End Function


      Public Shared Function GetNodeByText( _
              ByVal Nodes As TreeNodeCollection, ByVal sText As String, _
              Optional ByVal ThrowError As Boolean = True) As TreeNode
         Return GetByObject(Nodes, sText, "Text", ThrowError)
      End Function

      Public Shared Function GetNodeByPath( _
              ByVal Nodes As TreeNodeCollection, ByVal sPath As String, _
              Optional ByVal ThrowError As Boolean = True) As TreeNode
         If Nodes.Count = 0 Then Throw New ArgumentOutOfRangeException("Nodes", Nodes, _
             "TreeNodeCollection leer")
         Dim ndParent As TreeNode = Nodes(0).Parent
         Dim FullPathes As New List(Of String)
         Dim PathBuilder As New ObsoleteStringList
         If ndParent IsNot Nothing Then
            PathBuilder.Add(ndParent.FullPath)
         End If
         For Each Segment As String In sPath.Split("\"c)
            PathBuilder.Add(Segment)
            FullPathes.Add(PathBuilder.ToString("\"))
         Next
         Return GetByObjects(Nodes, FullPathes, "FullPath", ThrowError)

         Return GetByObjects(Nodes, sPath.Split("\"c), "Text", ThrowError)
      End Function

      Public Shared Function GetByObject( _
              ByVal Nodes As TreeNodeCollection, ByVal oKey As Object, _
              ByVal PropName As String, _
              Optional ByVal ThrowError As Boolean = True) As TreeNode
         For Each GetByObject In Nodes
            If CallByName(GetByObject, PropName, CallType.Get).Equals(oKey) Then Exit Function
         Next
         If ThrowError Then
            Throw New ArgumentOutOfRangeException("oKey", oKey, _
                "no Treenode found with that Key!")
         End If
         Return Nothing
      End Function

      Public Shared Function GetByObjects( _
              ByVal Nodes As TreeNodeCollection, ByVal Keys As IList, _
              ByVal PropName As String, _
              Optional ByVal ThrowError As Boolean = True) As TreeNode
         GetByObjects = Nothing
         Dim ndLoop As TreeNode
         Dim TargetKey As Object = Keys(Keys.Count - 1)
         'einen Node suchen, dessen Wert der spezifische Eigenschaft in Keys enthalten ist
         For Each ndLoop In Nodes
            If Keys.Contains(CallByName(ndLoop, PropName, CallType.Get)) Then
               GoTo FOUND_ENTRY
            End If
         Next
         If ThrowError Then
            Throw New ArgumentOutOfRangeException("TargetKey", TargetKey, _
                "no Treenode found with that Key!")
         End If
         Return Nothing
FOUND_ENTRY:
         If CallByName(ndLoop, PropName, CallType.Get).Equals(TargetKey) Then Return ndLoop
         For Each Segment As Object In Keys
            GetByObjects = GetByObject(Nodes, Segment, PropName, ThrowError)
            If GetByObjects Is Nothing Then Exit Function
            Nodes = GetByObjects.Nodes
         Next
      End Function

#End Region 'Public Shared Stuff

   End Class
End Namespace
